#include "stars.h"
#include "config.h"
#include <cstdlib>


StarsField::StarsField(unsigned speed)
: speed_(speed)
, field_(RequiredCapacity())
{
	color_.red = color_.green = color_.blue =
		std::min(65535u, 25000u + speed * speed * 2048);
	X11::Inst().AllocColor(&color_);
}


void StarsField::Scroll()
{
	// Adapt to window resize.
	if (field_.Capacity() < RequiredCapacity()) {
		field_.Grow(RequiredCapacity() - field_.Capacity());
	}

	// Scroll existing stars, destroying out of window ones.
	DrawAll(X11::Inst().GetBlack());
	for (FieldCtr::size_type v = 0; v < field_.Size(); ++v) {
		field_[v].AddY(speed_);
	}
	while (field_.Size() > 0 && field_[0].YOut()) {
		field_.Pop();
	}
	DrawAll(color_.pixel);

	// Create one star in an (empty) top line.
	const Coord star (std::rand() % X11::Inst().WindowWidth(),
	                  std::rand() % speed_);
	field_.Push(star);
	X11::Inst().SetForeground(color_.pixel);
	X11::Inst().DrawPoint(star);
}


void StarsField::DrawAll(X11::Color color) const
{
	if (field_.FirstRowSize() > 0) {
		X11::Inst().SetForeground(color);
		X11::Inst().DrawPoints(field_.FirstRow(),
		                       field_.FirstRowSize());
		if (field_.SecondRowSize() > 0) {
			X11::Inst().DrawPoints(field_.SecondRow(),
			                       field_.SecondRowSize());
		}
	}
}


void ScrollStarFields(std::vector<StarsField> * stars_fields)
{
	for (std::vector<StarsField>::size_type i
	     (Details::max - Details::Instance().Get());
	     i < stars_fields->size(); ++i) {
		(*stars_fields)[i].Scroll();
	}
}
